
/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Utility MACRO */
#define sqr(x) ((x)*(x))
#define deg2rad(deg) (M_PI*(deg)/180)

/* The name is missleading */
/* Animated transformation */
/* Used between mousemotion and moveposittion */
struct Viewpoint {
	GLfloat center[3];
	GLfloat alpha;
	GLfloat beta;
	GLfloat gamma;
	GLfloat zoom;
};
struct Viewpoint init_viewpoint = { { 0, 0, 0}, 0, 0, 0, 1}; 
struct Viewpoint viewpoint = { { 0, 0, 0}, 0, 0, 0, 1}; 


/* Mouse movement structure */
/* Used between mouse and mousemotion to track mouse movements */
enum Motion_mode { NONE=0, ROTATE, DIST,  PAN };

struct Motion { 
	enum Motion_mode mode;
	int startx;
	int starty;
};

struct Motion motion = { NONE, 0, 0 };

int perspective=0;

/* Draw standard Axes */
void axes() {
	int i;

	/* Data for just the standandar X axis */
	/* Others will be permutation on coordinates */
	GLfloat axis[][3] =   {  { -1  , 0    , 0     },
						     { 1   , 0    , 0     },
						     { 0.9 , 0.1  , 0     },
						     { 0.9 , -0.1 , 0     },
							 { 1   , 0    , 0     },
						     { 0.9 , 0    , 0.1   },
						     { 0.9 , 0    ,  -0.1 },
						     { 1   , 0    ,   0   } };
			
	/* X axis */		
	glColor3ub(255, 0, 0);
	glBegin(GL_LINE_STRIP);
	//for(i=0;i<8;i++) glVertex3f(axis[i][0],axis[i][1],axis[i][2]);
	for(i=0;i<8;i++) glVertex3fv(axis[i]);
	glEnd();
	
	/* Y axis */
	glColor3ub(0, 255, 0);
	glBegin(GL_LINE_STRIP);
	for(i=0;i<8;i++) glVertex3f(axis[i][2],axis[i][0],axis[i][1]);
	glEnd();
	
	/* Z axis */
	glColor3ub(0, 0, 255);
	glBegin(GL_LINE_STRIP);
	for(i=0;i<8;i++) glVertex3f(axis[i][1],axis[i][2],axis[i][0]);
	glEnd();
	
}


/* Animated model move */
void moveposition() {
	/* modelview matric */
	glMatrixMode(GL_MODELVIEW);
	
	/* Translate along (x,0,-z), (0,y,0) , (-x,0,-z) */
	glTranslatef(viewpoint.center[0]-viewpoint.center[2], 
	             viewpoint.center[1],
				 viewpoint.center[2]-viewpoint.center[0]
				 );
				 
	/* Rotate on (x,0,-z) */
	glRotatef(viewpoint.beta, cos(M_PI_4), 0, -sin(M_PI_4) );
	
	/* Rotate on (0,y,0) */
	glRotatef(viewpoint.alpha, 0, 1, 0 );
	
	/* Zoom scale */
	glScalef( viewpoint.zoom, viewpoint.zoom, viewpoint.zoom);
	    
}

/* Mause Click Callback */
void mouse(int button, int state, int x, int y) {
	/* A mouse button pressed */
	if(state==GLUT_DOWN) {
		/* Save position */
		motion.startx=x;
		motion.starty=y;
		
		/* Which button? */
		switch (button) {
			/* Left -> Rotate */
			case GLUT_LEFT_BUTTON:
				motion.mode=ROTATE;
				break;
			/* Middle -> zoom */
			case GLUT_MIDDLE_BUTTON:
				motion.mode=DIST;
				break;
			/* Right -> Pan */
			case GLUT_RIGHT_BUTTON:
				motion.mode=PAN;
				break;
			default:
				motion.mode=NONE;
		}
	} else { 
		motion.mode=NONE;
	}
		
}

void mousemotion(int x, int y) {
	/* Current mode? */
	switch (motion.mode) {
		
		/* Rotate, modify alfa and beta */
		case ROTATE:
			viewpoint.alpha+=(GLfloat) ((x-motion.startx) % 360);
			viewpoint.beta+=(GLfloat) ((y-motion.starty) % 360);
			break;
		/* Zoom */
		case DIST:
			viewpoint.zoom+=4.0 * ((GLfloat) (y-motion.starty))/((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			break;
		/* Pan */
		case PAN:
			viewpoint.center[0]+=4.0 * ((GLfloat) (x-motion.startx)) / ((GLfloat) glutGet(GLUT_WINDOW_WIDTH));
			/*if(glutGetModifiers()==GLUT_ACTIVE_SHIFT) { // Non funge!!!
				viewpoint.center[2]-=4.0 * ((GLfloat) (y-motion.starty)) / ((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			} else { */
			
				viewpoint.center[1]-=4.0 * ((GLfloat) (y-motion.starty)) / ((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			
			//}
			
			break;
		default:
			break;
	}
	
	/* Save current position */
	motion.startx=x;
	motion.starty=y;
	
	/* Redisplay */
	glutPostRedisplay();
}

/* Eye position */
void position() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt( 2, 2, 2, 0, 0, 0, 0, 1, 0 );
}


/* Current model */
void model() {
	GLfloat light_pos[]={ 2.0, 2.0, 3.0 };

	/* Fiat Lux */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos );
	
	/* Animated move */
	moveposition();
	
	/* Model */
	glColor3ub(100, 100, 100);
	glutSolidTeapot(1);
	
	/* Goto darkness */
	glDisable(GL_LIGHTING);
}

void display(void) {
		
	/* Clear Buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	
	/* Position Eye */
	position();

	/* Draw axes */
	axes();
	
	/* Draw model */
	model();
	
	/* SwapBuffer */
	glutSwapBuffers();
}



void reshape(int w, int h) {	
	/* Current aspect ratio */
	float ratio=((float) w) / ((float) h);

	glViewport(0, 0, w, h);

	/* Projection Matrix */
	glMatrixMode(GL_PROJECTION);
	
	/* Load identity */
	glLoadIdentity();
	
	/* The projection */
	if (!perspective) {
		/* Orthografic */
		/* Keep 1:1 ratio, shortest side must be at least [-1,1] */
		if (ratio>1) glOrtho( -ratio, ratio, -1, 1, 1, 50);
		else glOrtho( -1, 1, -1/ratio, 1/ratio, 1, 50);
	} else {
		/* OR Perspective */
		gluPerspective(35, ratio, 1, 50 );
	}
	
	/* Redraw this window */
	glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y) {
	
	static int fullscreen=0;
	static int old_w,old_h;
	static int old_x,old_y;
	switch(key) {
		/* Q or esc: Quit */
		case 27:
		case 'q':
		case 'Q':
			exit(0);
			break;
		/* F: Fullscreen toogle */
		case 'f':
		case 'F':
			/* Not in full screen */
			if (!fullscreen) {
				/* Save window size and position */
				old_w=glutGet(GLUT_WINDOW_WIDTH);			
				old_h=glutGet(GLUT_WINDOW_HEIGHT);
				old_x=glutGet(GLUT_WINDOW_X);
				old_y=glutGet(GLUT_WINDOW_Y);
				/* Go fullscreen */
				glutFullScreen();
				fullscreen=1;
			} else {
				/* Set window size and shape to original values */
				glutReshapeWindow(old_w, old_h);
				glutPositionWindow(old_x, old_y);
				fullscreen=0;
			}
			glutPostRedisplay();
			break;
		/* p: Perspective */
		case 'p':
		case 'P':
			perspective=!perspective;
			/* Call reshape */
			reshape(glutGet(GLUT_WINDOW_WIDTH),  glutGet(GLUT_WINDOW_HEIGHT));
			glutPostRedisplay();
			break;
		/* o,r,c: Reset view */
		case 'o':
		case 'O':
		case 'r':
		case 'R':
		case 'c':
		case 'C':
			memcpy(&viewpoint,&init_viewpoint,sizeof(viewpoint));
			glutPostRedisplay();
			break;
	}
	
}

int main (int argc, const char** argv) {

	/* Init Glut */
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	
	/* Main Window */
    glutInitWindowSize(800, 600);
    glutInitWindowPosition(0, 0);
	glutCreateWindow("Glutviewer");
	
	/* Set Callbacks */
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(mousemotion);
    
	/* Glut Main Loop */
	glutMainLoop();

	/* Never reached, keep compiler happy */
	return 0;
}
